@@ -1,4 +1,4 @@
-/* $Id: blast_query_info.h,v 1.6 2011/11/30 13:59:31 kazimird Exp $
+/* $Id: blast_query_info.h,v 1.9 2016/06/20 15:49:13 fukanchi Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -55,6 +55,8 @@ typedef struct BlastContextInfo {
                               This field should be set only by the setup code
                               and read by subsequent stages of the BLAST search
                               */
+    Boolean has_pair;       /**< If true, this query has a pair in the contexts
+                                 for the next sequence (for mapping) */
 } BlastContextInfo;
 
 /** Forward declaration of SPHIQueryInfo */
@@ -69,6 +71,8 @@ typedef struct BlastQueryInfo {
     BlastContextInfo * contexts; /**< Information per context */
     Uint4 max_length;    /**< Length of the longest among the concatenated
                             queries */
+    Uint4 min_length;    /**< Length of the shortest among the concatenated
+                            queries */
     struct SPHIQueryInfo* pattern_info; /**< Counts of PHI BLAST pattern
                                       occurrences, used in PHI BLAST only. */
 } BlastQueryInfo;
@@ -84,7 +88,7 @@ BlastQueryInfo* BlastQueryInfoFree(BlastQueryInfo* query_info);
 
 /** Duplicates the query information structure */
 NCBI_XBLAST_EXPORT
-BlastQueryInfo* BlastQueryInfoDup(BlastQueryInfo* query_info);
+BlastQueryInfo* BlastQueryInfoDup(const BlastQueryInfo* query_info);
 
 /** Given a context from BLAST engine core, return the query index.
  * @param context Context saved in a BlastHSP structure [in]
@@ -191,7 +195,7 @@ QueryInfo_GetSeqBufLen(const BlastQueryInfo* qinfo);
  * @return Allocated array.
  */
 NCBI_XBLAST_EXPORT
-Int4 * ContextOffsetsToOffsetArray(BlastQueryInfo* info);
+Int4 * ContextOffsetsToOffsetArray(const BlastQueryInfo* info);
 
 
 /** Copy the context query offsets from an array of Int4, allocating
