@@ -1,4 +1,4 @@
-/* $Id: lookup_wrap.c,v 1.34 2012/05/21 15:56:03 kazimird Exp $
+/* $Id: lookup_wrap.c,v 1.37 2016/07/01 15:49:12 fukanchi Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -35,11 +35,6 @@
  * megablast lookup table, etc.
  */
 
-#ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] = 
-    "$Id: lookup_wrap.c,v 1.34 2012/05/21 15:56:03 kazimird Exp $";
-#endif /* SKIP_DOXYGEN_PROCESSING */
-
 #include <algo/blast/core/lookup_wrap.h>
 #include <algo/blast/core/blast_aalookup.h>
 #include <algo/blast/core/blast_nalookup.h>
@@ -54,7 +49,8 @@ Int2 LookupTableWrapInit(BLAST_SequenceBlk* query,
         const QuerySetUpOptions* query_options,
         BlastSeqLoc* lookup_segments, BlastScoreBlk* sbp, 
         LookupTableWrap** lookup_wrap_ptr, const BlastRPSInfo *rps_info,
-        Blast_Message* *error_msg)
+        Blast_Message* *error_msg,
+        BlastSeqSrc* seqsrc)
 {
    Int2 status = 0;
    LookupTableWrap* lookup_wrap;
@@ -121,7 +117,8 @@ Int2 LookupTableWrapInit(BLAST_SequenceBlk* query,
              BlastMBLookupTableNew(query, lookup_segments, 
                                (BlastMBLookupTable* *) &(lookup_wrap->lut), 
                                lookup_options, query_options,
-                               num_table_entries, lut_width);
+                               num_table_entries, lut_width,
+                               seqsrc);
           }
           else if (lookup_wrap->lut_type == eSmallNaLookupTable) {
              status = BlastSmallNaLookupTableNew(query, lookup_segments,
@@ -143,6 +140,13 @@ Int2 LookupTableWrapInit(BLAST_SequenceBlk* query,
       ASSERT( lookup_wrap->lut_type != eMixedMBLookupTable );
       break;
 
+   case eNaHashLookupTable:
+           status = BlastNaHashLookupTableNew(query, lookup_segments,
+                             (BlastNaHashLookupTable**) &(lookup_wrap->lut), 
+                             lookup_options, query_options, seqsrc);
+       break;
+
+
    case ePhiLookupTable: case ePhiNaLookupTable:
        {
            const Boolean kIsDna = 
@@ -210,6 +214,11 @@ LookupTableWrap* LookupTableWrapFree(LookupTableWrap* lookup)
          BlastNaLookupTableDestruct((BlastNaLookupTable*)lookup->lut);
       break;
 
+   case eNaHashLookupTable:
+      lookup->lut = (void*) 
+         BlastNaHashLookupTableDestruct((BlastNaHashLookupTable*)lookup->lut);
+      break;
+
    case eAaLookupTable:
       lookup->lut = (void*) 
          BlastAaLookupTableDestruct((BlastAaLookupTable*)lookup->lut);
@@ -251,6 +260,10 @@ Int4 GetOffsetArraySize(LookupTableWrap* lookup)
       offset_array_size = OFFSET_ARRAY_SIZE + 
          ((BlastNaLookupTable*)lookup->lut)->longest_chain;
       break;
+   case eNaHashLookupTable:
+      offset_array_size = OFFSET_ARRAY_SIZE + 
+         ((BlastNaHashLookupTable*)lookup->lut)->longest_chain;
+      break;
    default:
       offset_array_size = OFFSET_ARRAY_SIZE;
       break;
