@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_CONNECTOR__H
 #define CONNECT___NCBI_CONNECTOR__H
 
-/* $Id: ncbi_connector.h,v 6.19 2012/05/07 15:39:33 kazimird Exp $
+/* $Id: ncbi_connector.h,v 6.21 2016/03/04 01:49:14 fukanchi Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -51,7 +51,7 @@ extern "C" {
 
 
 struct SConnectorTag;
-typedef struct SConnectorTag* CONNECTOR;  /* connector handle */
+typedef struct SConnectorTag* CONNECTOR;  /**< connector handle */
 
 
 /* Function type definitions for the connector method table.
@@ -66,25 +66,24 @@ typedef struct SConnectorTag* CONNECTOR;  /* connector handle */
  */
 
 
-/* Get the name of the connector (may be NULL on error)
+/** Get the name of the connector (may be NULL on error)
  */
 typedef const char* (*FConnectorGetType)
 (CONNECTOR       connector
  );
 
 
-/* Get the human readable connector's description (may be NULL on error)
+/** Get the human readable connector's description (may be NULL on error)
  */
 typedef       char* (*FConnectorDescr)
 (CONNECTOR       connector
  );
 
 
-/* Open connection. Used to setup all related data structures,
+/** Open connection.  Used to setup all related data structures,
  * but not necessarily has to actually open the data channel.
- * NOTE: Regardless of the returned status, the connection is
- *       considered open (so this call does not get repeated)
- *       when the call completes.
+ * @note  Regardless of the returned status, the connection is considered open
+ *        (so this call does not get repeated) when the call returns.
  */
 typedef EIO_Status (*FConnectorOpen)
 (CONNECTOR       connector,
@@ -92,11 +91,11 @@ typedef EIO_Status (*FConnectorOpen)
  );
 
 
-/* Wait until either read or write (dep. on the "direction" value) becomes
+/** Wait until either read or write (dep. on the "direction" value) becomes
  * available, or until "timeout" expires, or until error occurs.
- * NOTE 1:  FConnectorWait is guaranteed to be called after FConnectorOpen,
- *          and only if FConnectorOpen returned eIO_Success.
- * NOTE 2:  "event" is guaranteed to be either eIO_Read or eIO_Write.
+ * @note  FConnectorWait() is guaranteed to be called after FConnectorOpen(),
+ *        and only if the latter succeeded (returned eIO_Success).
+ * @note  The passed "event" is guaranteed to be either eIO_Read or eIO_Write.
  */
 typedef EIO_Status (*FConnectorWait)
 (CONNECTOR       connector,
@@ -105,10 +104,11 @@ typedef EIO_Status (*FConnectorWait)
  );
 
 
-/* The passed "n_written" is always non-NULL, and "*n_written" is always zero.
+/** Write to connector.
+ * The passed "n_written" is always non-NULL, and "*n_written" is always zero.
  * The number of bytes actually written gets returned in "*n_written".
- * It may not return "eIO_Success" if no data at all have been written.
- * NOTE:  FConnectorWrite is guaranteed to be called after FConnectorOpen,
+ * It may not return eIO_Success if no data at all have been written.
+ * @note  FConnectorWrite() is guaranteed to be called after FConnectorOpen(),
  *        and only if the latter succeeded (returned eIO_Success).
  */
 typedef EIO_Status (*FConnectorWrite)
@@ -120,8 +120,8 @@ typedef EIO_Status (*FConnectorWrite)
  );
 
 
-/* Flush yet unwritten output data, if any.
- * NOTE:  FConnectorFlush is guaranteed to be called after FConnectorOpen,
+/** Flush yet unwritten output data, if any.
+ * @note  FConnectorFlush() is guaranteed to be called after FConnectorOpen(),
  *        and only if the latter succeeded (returned eIO_Success).
  */
 typedef EIO_Status (*FConnectorFlush)
@@ -130,10 +130,11 @@ typedef EIO_Status (*FConnectorFlush)
  );
 
 
-/* The passed "n_read" is always non-NULL, and "*n_read" is always zero.
+/** Read from connector.
+ * The passed "n_read" is always non-NULL, and "*n_read" is always zero.
  * The number of bytes actually read bytes gets returned in "*n_read".
  * It may not return eIO_Success if no data at all have been read.
- * NOTE:  FConnectorRead is guaranteed to be called after FConnectorOpen,
+ * @note  FConnectorRead() is guaranteed to be called after FConnectorOpen(),
  *        and only if the latter succeeded (returned eIO_Success).
  */
 typedef EIO_Status (*FConnectorRead)
@@ -144,12 +145,13 @@ typedef EIO_Status (*FConnectorRead)
  const STimeout* timeout
  );
 
-/* Obtain last I/O completion code from the transport level (connector).
- * NOTE 1:  FConnectorStatus is guaranteed to be called after FConnectorOpen,
- *          and only if the latter succeeded (returned eIO_Success).
- * NOTE 2:  "direction" is guaranteed to be either eIO_Read or eIO_Write.
- * NOTE 3:  Should return eIO_Success in case of inexistent (incomplete)
- *          low level transport, if any.
+
+/** Obtain last I/O completion code from the transport level (connector).
+ * @note  FConnectorStatus() is guaranteed to be called after FConnectorOpen(),
+ *        and only if the latter succeeded (returned eIO_Success).
+ * @note  "direction" is guaranteed to be either eIO_Read or eIO_Write.
+ * @note  Should return eIO_Success in case of inexistent (incomplete)
+ *        low level transport, if any.
  */
 typedef EIO_Status (*FConnectorStatus)
 (CONNECTOR       connector,
@@ -157,11 +159,11 @@ typedef EIO_Status (*FConnectorStatus)
  );
           
 
-/* Close data link (if any) and cleanup related data structures.
- * NOTE 1:  FConnectorClose is guaranteed to be called after FConnectorOpen,
- *          and only if the latter succeeded (returned eIO_Success).
- * NOTE 2:  FConnectorFlush gets called before FConnectorClose automatically.
- * NOTE 3:  It may return eIO_Closed to indicate an unusual close condition.
+/** Close data link (if any) and cleanup related data structures.
+ * @note  FConnectorClose() is guaranteed to be called after FConnectorOpen(),
+ *        and only if the latter succeeded (returned eIO_Success).
+ * @note  FConnectorFlush() gets called before FConnectorClose() automatically.
+ * @note  It may return eIO_Closed to indicate an unusual close condition.
  */
 typedef EIO_Status (*FConnectorClose)
 (CONNECTOR       connector,
@@ -169,22 +171,24 @@ typedef EIO_Status (*FConnectorClose)
  );
 
 
-/* Standard set of connector methods to handle a connection (corresponding
+/** Standard set of connector methods to handle a connection (corresponding
  * connectors are also here), part of connection handle("CONN").
+ * @sa
+ *  CONN
  */
 typedef struct {
-    FConnectorGetType   get_type; CONNECTOR c_get_type;
-    FConnectorDescr     descr;    CONNECTOR c_descr;
-    FConnectorOpen      open;     CONNECTOR c_open;
-    FConnectorWait      wait;     CONNECTOR c_wait;
-    FConnectorWrite     write;    CONNECTOR c_write;
-    FConnectorFlush     flush;    CONNECTOR c_flush;
-    FConnectorRead      read;     CONNECTOR c_read;
-    FConnectorStatus    status;   CONNECTOR c_status;
-    FConnectorClose     close;    CONNECTOR c_close;
-    const STimeout*     default_timeout; /* default timeout pointer     */
-    STimeout            default_tmo;     /* storage for default_timeout  */
-    CONNECTOR           list;
+    FConnectorGetType get_type;  CONNECTOR c_get_type;
+    FConnectorDescr   descr;     CONNECTOR c_descr;
+    FConnectorOpen    open;      CONNECTOR c_open;
+    FConnectorWait    wait;      CONNECTOR c_wait;
+    FConnectorWrite   write;     CONNECTOR c_write;
+    FConnectorFlush   flush;     CONNECTOR c_flush;
+    FConnectorRead    read;      CONNECTOR c_read;
+    FConnectorStatus  status;    CONNECTOR c_status;
+    FConnectorClose   close;     CONNECTOR c_close;
+    const STimeout*   default_timeout;  /**< default timeout pointer     */
+    STimeout          default_tmo;      /**< storage for default_timeout */
+    CONNECTOR         list;
 } SMetaConnector;
 
 
@@ -207,7 +211,7 @@ typedef struct {
     } while (0)
 
 
-/* Insert a connector in the beginning of the connection's list of connectors.
+/** Insert a connector in the beginning of the connection's list of connectors.
  * Calls connector's FSetupVTable, which must be defined.
  */
 extern NCBI_XCONNECT_EXPORT EIO_Status METACONN_Add
@@ -216,7 +220,7 @@ extern NCBI_XCONNECT_EXPORT EIO_Status METACONN_Add
  );
 
 
-/* Delete given "connector" all its descendants (all connectors if
+/** Delete given "connector" all its descendants (all connectors if
  * "connector" is 0) from the connections's list of connectors.
  * FDestroy (if defined) gets called for each removed connector.
  */
@@ -226,14 +230,14 @@ extern NCBI_XCONNECT_EXPORT EIO_Status METACONN_Remove
  );
 
 
-/* Upcall on request to setup virtual function table (called from connection).
+/** Upcall on request to setup virtual function table (called from connection).
  */
 typedef void (*FSetupVTable)
 (CONNECTOR       connector
  );
 
 
-/* Destroy connector and its data handle.  This is NOT a close request!
+/** Destroy connector and its data handle.  This is NOT a close request!
  * Should not to be used on open connectors (that is, for those
  * FConnectorClose must be called prior to this call).
  */
@@ -242,14 +246,14 @@ typedef void (*FDestroy)
  );
 
 
-/* Connector specification.
+/** Connector specification.
  */
 typedef struct SConnectorTag {
-    SMetaConnector* meta;     /* back link to original meta   */
-    FSetupVTable    setup;    /* init meta, may not be NULL   */
-    FDestroy        destroy;  /* destroys handle, can be NULL */
-    void*           handle;   /* data handle of the connector */
-    CONNECTOR       next;     /* linked list                  */
+    SMetaConnector* meta;     /**< back link to original meta   */
+    FSetupVTable    setup;    /**< init meta, may not be NULL   */
+    FDestroy        destroy;  /**< destroys handle, can be NULL */
+    void*           handle;   /**< data handle of the connector */
+    CONNECTOR       next;     /**< linked list                  */
 } SConnector;
 
 
