@@ -1,4 +1,4 @@
-/* $Id: ncbi_service.c,v 6.128 2012/02/01 17:49:38 kazimird Exp $
+/* $Id: ncbi_service.c,v 6.150 2016/01/01 00:39:17 fukanchi Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -34,6 +34,9 @@
 #include "ncbi_dispd.h"
 #include "ncbi_lbsmd.h"
 #include "ncbi_local.h"
+#ifdef NCBI_CXX_TOOLKIT
+#  include "ncbi_lbosp.h"
+#endif
 #include "ncbi_priv.h"
 #include <ctype.h>
 #include <stdlib.h>
@@ -41,20 +44,24 @@
 
 #define NCBI_USE_ERRCODE_X   Connect_Service
 
+
+/*
+ * FIXME FIXME FIXME FIXME FIXME ---->
+ *
+ * NOTE: For fSERV_ReverseDns lookups the following rules apply to "skip"
+ *       contents:  a service would not be selected if there is a same-named
+ *       entry with matching host[:port] is found in the "skip" list, or
+ *       there is a nameless...
+ * NOTE:  Lookup by mask cancels fSERV_ReverseDns mode, if both are used.
+ */
+
+
 #define CONN_SERVICE_NAME  DEF_CONN_REG_SECTION "_" REG_CONN_SERVICE_NAME
 
 
-/* Use a property that "&a" for an array is equivalent to just "a", but for
- * pointers it will increase indirection, so SizeOf() will always be 1. */
-#define SizeOf(arr)  (sizeof(arr) / sizeof((arr)[0]))
-
-
 static ESwitch s_Fast = eOff;
 
 
-static TNCBI_BigCount s_FWPorts[1024 / sizeof(TNCBI_BigCount)] = { 0 };
-
-
 ESwitch SERV_DoFastOpens(ESwitch on)
 {
     ESwitch retval = s_Fast;
@@ -64,32 +71,8 @@ ESwitch SERV_DoFastOpens(ESwitch on)
 }
 
 
-int/*bool*/ SERV_AddFirewallPort(unsigned short port)
-{
-    unsigned int n = port / (sizeof(s_FWPorts[0]) << 3);
-    unsigned int m = port % (sizeof(s_FWPorts[0]) << 3);
-    if ((size_t) n < SizeOf(s_FWPorts)) {
-        s_FWPorts[n] |= (TNCBI_BigCount) 1 << m;
-        return 1/*true*/;
-    }
-    return 0/*false*/;
-}
-
-
-int/*bool*/ SERV_IsFirewallPort(unsigned short port)
-{
-    unsigned int n = port / (sizeof(s_FWPorts[0]) << 3);
-    unsigned int m = port % (sizeof(s_FWPorts[0]) << 3);
-    if ((size_t) n < SizeOf(s_FWPorts)
-        &&  (s_FWPorts[n] & ((TNCBI_BigCount) 1 << m))) {
-        return 1/*true*/;
-    }
-    return 0/*false*/;
-}
-
-
-static char* s_ServiceName(const char* service,
-                           int/*bool*/ ismask, size_t depth)
+static char* x_ServiceName(const char* service,
+                           int/*bool*/ ismask, unsigned int depth)
 {
     char   buf[128];
     char   srv[128];
@@ -99,8 +82,8 @@ static char* s_ServiceName(const char* service,
     if (depth > 7) {
         assert(service  &&  *service);
         CORE_LOGF_X(7, eLOG_Error,
-                    ("[%s]  Maximal service name recursion depth reached: %lu",
-                     service, (unsigned long) depth));
+                    ("[%s]  Maximal service name recursion depth reached: %u",
+                     service, depth));
         return 0/*failure*/;
     }
     len = 0;
@@ -129,21 +112,31 @@ static char* s_ServiceName(const char* service,
             s = srv;
         }
         if (*s  &&  strcasecmp(s, service) != 0)
-            return s_ServiceName(s, ismask, ++depth);
+            return x_ServiceName(s, ismask, depth + 1);
     }
     return strdup(service);
 }
 
 
-char* SERV_ServiceName(const char* service)
+static char* s_ServiceName(const char* service, int/*bool*/ ismask)
 {
-    return s_ServiceName(service, 0, 0);
+    char* retval;
+    CORE_LOCK_READ;
+    retval = x_ServiceName(service, ismask, 0);
+    CORE_UNLOCK;
+    return retval;
 }
 
 
-static int/*bool*/ s_AddSkipInfo(SERV_ITER   iter,
-                                 const char* name,
-                                 SSERV_Info* info)
+char* SERV_ServiceName(const char* service)
+{
+    return s_ServiceName(service, 0);
+}
+
+
+static int/*bool*/ s_AddSkipInfo(SERV_ITER      iter,
+                                 const char*    name,
+                                 SSERV_InfoCPtr info)
 {
     size_t n;
     assert(name);
@@ -155,18 +148,18 @@ static int/*bool*/ s_AddSkipInfo(SERV_ITER   iter,
             /* Replace older version */
             if (iter->last == iter->skip[n])
                 iter->last  = info;
-            free(iter->skip[n]);
+            free((void*) iter->skip[n]);
             iter->skip[n] = info;
             return 1;
         }
     }
     if (iter->n_skip == iter->a_skip) {
-        SSERV_Info** temp;
+        SSERV_InfoCPtr* temp;
         n = iter->a_skip + 10;
-        temp = (SSERV_Info**)
+        temp = (SSERV_InfoCPtr*)
             (iter->skip
-             ? realloc(iter->skip, n * sizeof(*temp))
-             : malloc (            n * sizeof(*temp)));
+             ? realloc((void*) iter->skip, n * sizeof(*temp))
+             : malloc (                    n * sizeof(*temp)));
         if (!temp)
             return 0;
         iter->skip = temp;
@@ -190,27 +183,32 @@ static int/*bool*/ s_IsMapperConfigured(const char* service, const char* key)
 }
 
 
-static SERV_ITER s_Open(const char*          service,
-                        unsigned/*bool*/     ismask,
-                        TSERV_Type           types,
-                        unsigned int         preferred_host,
-                        unsigned short       preferred_port,
-                        double               preference,
-                        const SConnNetInfo*  net_info,
-                        const SSERV_InfoCPtr skip[],
-                        size_t               n_skip,
-                        unsigned/*bool*/     external,
-                        const char*          arg,
-                        const char*          val,
-                        SSERV_Info**         info,
-                        HOST_INFO*           host_info)
+static SERV_ITER x_Open(const char*         service,
+                        unsigned/*bool*/    ismask,
+                        TSERV_Type          types,
+                        unsigned int        preferred_host,
+                        unsigned short      preferred_port,
+                        double              preference,
+                        const SConnNetInfo* net_info,
+                        SSERV_InfoCPtr      skip[],
+                        size_t              n_skip,
+                        unsigned/*bool*/    external,
+                        const char*         arg,
+                        const char*         val,
+                        SSERV_Info**        info,
+                        HOST_INFO*          host_info)
 {
-    int/*bool*/ do_lbsmd = -1/*unassigned*/, do_dispd = -1/*unassigned*/;
+    int/*bool*/
+        do_lbsmd = -1/*unassigned*/,
+#ifdef NCBI_CXX_TOOLKIT
+        do_lbos  = -1/*unassigned*/,
+#endif /*NCBI_CXX_TOOLKIT*/
+        do_dispd = -1/*unassigned*/;
     const SSERV_VTable* op;
     SERV_ITER iter;
     const char* s;
 
-    if (!(s = s_ServiceName(service, ismask, 0)))
+    if (!(s = s_ServiceName(service, ismask)))
         return 0;
     if (!(iter = (SERV_ITER) calloc(1, sizeof(*iter)))) {
         free((void*) s);
@@ -219,7 +217,6 @@ static SERV_ITER s_Open(const char*          service,
     assert(ismask  ||  *s);
 
     iter->name              = s;
-    iter->type              = types & fSERV_All;
     iter->host              = (preferred_host == SERV_LOCALHOST
                                ? SOCK_GetLocalHostAddress(eDefault)
                                : preferred_host);
@@ -229,10 +226,13 @@ static SERV_ITER s_Open(const char*          service,
                                :  0.01 * (preference > 100.0
                                           ? 100.0
                                           : preference));
+    iter->types             = types;
     if (ismask)
         iter->ismask        = 1;
     if (types & fSERV_IncludeDown)
         iter->ok_down       = 1;
+    if (types & fSERV_IncludeReserved)
+        iter->ok_reserved   = 1;
     if (types & fSERV_IncludeSuppressed)
         iter->ok_suppressed = 1;
     if (types & fSERV_ReverseDns)
@@ -272,10 +272,11 @@ static SERV_ITER s_Open(const char*          service,
         }
     }
     assert(n_skip == iter->n_skip);
+    iter->o_skip = iter->n_skip;
 
     if (net_info) {
         if (net_info->firewall)
-            iter->type |= fSERV_Firewall;
+            iter->types |= fSERV_Firewall;
         if (net_info->stateless)
             iter->stateless = 1;
         if (net_info->lb_disable)
@@ -284,15 +285,35 @@ static SERV_ITER s_Open(const char*          service,
         do_dispd = 0/*false*/;
     /* Ugly optimization not to access the registry more than necessary */
     if ((!s_IsMapperConfigured(service, REG_CONN_LOCAL_ENABLE)               ||
-         !(op = SERV_LOCAL_Open(iter, info, host_info)))                 &&
+         !(op = SERV_LOCAL_Open(iter, info, host_info)))
+
+        &&
         (!do_lbsmd                                                           ||
-         !(do_lbsmd= !s_IsMapperConfigured(service, REG_CONN_LBSMD_DISABLE)) ||
+         !(do_lbsmd = !s_IsMapperConfigured
+           (service, REG_CONN_LBSMD_DISABLE))                                ||
          !(op = SERV_LBSMD_Open(iter, info, host_info,
-                                !do_dispd                                    ||
-                                !(do_dispd = !s_IsMapperConfigured
-                                  (service, REG_CONN_DISPD_DISABLE)))))  &&
+                                (!do_dispd                                   ||
+                                 !(do_dispd = !s_IsMapperConfigured
+                                   (service, REG_CONN_DISPD_DISABLE)))
+#ifdef NCBI_CXX_TOOLKIT
+                                &&
+                                !(do_lbos = s_IsMapperConfigured
+                                  (service, REG_CONN_LBOS_ENABLE))
+#endif /*NCBI_CXX_TOOLKIT*/
+                                )))
+
+#ifdef NCBI_CXX_TOOLKIT
+        &&
+        (!do_lbos                                                            ||
+         (do_lbos < 0  &&  !(do_lbos = s_IsMapperConfigured
+                             (service, REG_CONN_LBOS_ENABLE)))               ||
+         !(op = SERV_LBOS_Open(iter, net_info, info)))
+#endif /*NCBI_CXX_TOOLKIT*/
+
+        && 
         (!do_dispd                                                           ||
-         !(do_dispd= !s_IsMapperConfigured(service, REG_CONN_DISPD_DISABLE)) ||
+         (do_dispd < 0  &&  !(do_dispd = !s_IsMapperConfigured
+                              (service, REG_CONN_DISPD_DISABLE)))            ||
          !(op = SERV_DISPD_Open(iter, net_info, info, host_info)))) {
         if (!do_lbsmd  &&  !do_dispd) {
             CORE_LOGF_X(1, eLOG_Error,
@@ -308,64 +329,6 @@ static SERV_ITER s_Open(const char*          service,
 }
 
 
-SERV_ITER SERV_OpenSimple(const char* service)
-{
-    SConnNetInfo* net_info = ConnNetInfo_Create(service);
-    SERV_ITER iter = SERV_Open(service, fSERV_Any, 0, net_info);
-    ConnNetInfo_Destroy(net_info);
-    return iter;
-}
-
-
-SERV_ITER SERV_Open(const char*         service,
-                    TSERV_Type          types,
-                    unsigned int        preferred_host,
-                    const SConnNetInfo* net_info)
-{
-    return s_Open(service, 0/*not mask*/, types,
-                  preferred_host, 0/*preferred_port*/, 0.0/*preference*/,
-                  net_info, 0/*skip*/, 0/*n_skip*/,
-                  0/*not external*/, 0/*arg*/, 0/*val*/,
-                  0/*info*/, 0/*host_info*/);
-}
-
-
-SERV_ITER SERV_OpenEx(const char*          service,
-                      TSERV_Type           types,
-                      unsigned int         preferred_host,
-                      const SConnNetInfo*  net_info,
-                      const SSERV_InfoCPtr skip[],
-                      size_t               n_skip)
-{
-    return s_Open(service, 0/*not mask*/, types,
-                  preferred_host, 0/*preferred_port*/, 0.0/*preference*/,
-                  net_info, skip, n_skip,
-                  0/*not external*/, 0/*arg*/, 0/*val*/,
-                  0/*info*/, 0/*host_info*/);
-}
-
-
-SERV_ITER SERV_OpenP(const char*          service,
-                     TSERV_Type           types,
-                     unsigned int         preferred_host,
-                     unsigned short       preferred_port,
-                     double               preference,
-                     const SConnNetInfo*  net_info,
-                     const SSERV_InfoCPtr skip[],
-                     size_t               n_skip,
-                     int/*bool*/          external,
-                     const char*          arg,
-                     const char*          val)
-{
-    return s_Open(service,
-                  service  &&  (!*service  ||  strpbrk(service, "?*")), types,
-                  preferred_host, preferred_port, preference,
-                  net_info, skip, n_skip,
-                  external, arg, val,
-                  0/*info*/, 0/*host_info*/);
-}
-
-
 static void s_SkipSkip(SERV_ITER iter)
 {
     size_t n;
@@ -373,18 +336,19 @@ static void s_SkipSkip(SERV_ITER iter)
         return;
     n = 0;
     while (n < iter->n_skip) {
-        SSERV_Info* temp = iter->skip[n];
+        SSERV_InfoCPtr temp = iter->skip[n];
         if (temp->time != NCBI_TIME_INFINITE
             &&  (!iter->time/*iterator reset*/
                  ||  ((temp->type != fSERV_Dns  ||  temp->host)
                       &&  temp->time < iter->time))) {
-            if (n < --iter->n_skip) {
-                memmove(iter->skip + n, iter->skip + n + 1,
-                        sizeof(*iter->skip)*(iter->n_skip - n));
+            if (--iter->n_skip > n) {
+                SSERV_InfoCPtr* ptr = iter->skip + n;
+                memmove((void*) ptr, (void*)(ptr + 1),
+                        (iter->n_skip - n) * sizeof(*ptr));
             }
             if (iter->last == temp)
                 iter->last  = 0;
-            free(temp);
+            free((void*) temp);
         } else
             n++;
     }
@@ -426,100 +390,197 @@ static SSERV_Info* s_GetNextInfo(SERV_ITER   iter,
 }
 
 
-static SSERV_Info* s_GetInfo(const char*          service,
-                             TSERV_Type           types,
-                             unsigned int         preferred_host,
-                             unsigned short       preferred_port,
-                             double               preference,
-                             const SConnNetInfo*  net_info,
-                             const SSERV_InfoCPtr skip[],
-                             size_t               n_skip,
-                             int /*bool*/         external,
-                             const char*          arg,
-                             const char*          val,
-                             HOST_INFO*           host_info)
+static SERV_ITER s_Open(const char*         service,
+                        unsigned/*bool*/    ismask,
+                        TSERV_Type          types,
+                        unsigned int        preferred_host,
+                        unsigned short      preferred_port,
+                        double              preference,
+                        const SConnNetInfo* net_info,
+                        SSERV_InfoCPtr      skip[],
+                        size_t              n_skip,
+                        int /*bool*/        external,
+                        const char*         arg,
+                        const char*         val,
+                        SSERV_Info**        info,
+                        HOST_INFO*          host_info)
 {
-    SSERV_Info* info = 0;
+    SSERV_Info* x_info;
+    SERV_ITER iter = x_Open(service, ismask, types,
+                            preferred_host, preferred_port, preference,
+                            net_info, skip, n_skip,
+                            external, arg, val,
+                            &x_info, host_info);
+    assert(!iter  ||  iter->op);
+    if (!iter)
+        x_info = 0;
+    else if (!x_info)
+        x_info = info ? s_GetNextInfo(iter, host_info, 1/*internal*/) : 0;
+    else if (x_info == (SSERV_Info*)(-1L)) {
+        SERV_Close(iter);
+        x_info = 0;
+        iter = 0;
+    }
+    if (info)
+        *info = x_info;
+    else if (x_info)
+        free(x_info);
+    return iter;
+}
+
+
+extern SERV_ITER SERV_OpenSimple(const char* service)
+{
+    SConnNetInfo* net_info = ConnNetInfo_Create(service);
+    SERV_ITER iter = s_Open(service, 0/*not mask*/, fSERV_Any,
+                            SERV_ANYHOST,
+                            0/*preferred_port*/, 0.0/*preference*/,
+                            net_info, 0/*skip*/, 0/*n_skip*/,
+                            0/*not external*/, 0/*arg*/, 0/*val*/,
+                            0/*info*/, 0/*host_info*/);
+    ConnNetInfo_Destroy(net_info);
+    return iter;
+}
+
+
+extern SERV_ITER SERV_Open(const char*         service,
+                           TSERV_Type          types,
+                           unsigned int        preferred_host,
+                           const SConnNetInfo* net_info)
+{
+    return s_Open(service, 0/*not mask*/, types,
+                  preferred_host, 0/*preferred_port*/, 0.0/*preference*/,
+                  net_info, 0/*skip*/, 0/*n_skip*/,
+                  0/*not external*/, 0/*arg*/, 0/*val*/,
+                  0/*info*/, 0/*host_info*/);
+}
+
+
+extern SERV_ITER SERV_OpenEx(const char*         service,
+                             TSERV_Type          types,
+                             unsigned int        preferred_host,
+                             const SConnNetInfo* net_info,
+                             SSERV_InfoCPtr      skip[],
+                             size_t              n_skip)
+{
+    return s_Open(service, 0/*not mask*/, types,
+                  preferred_host, 0/*preferred_port*/, 0.0/*preference*/,
+                  net_info, skip, n_skip,
+                  0/*not external*/, 0/*arg*/, 0/*val*/,
+                  0/*info*/, 0/*host_info*/);
+}
+
+
+SERV_ITER SERV_OpenP(const char*         service,
+                     TSERV_Type          types,
+                     unsigned int        preferred_host,
+                     unsigned short      preferred_port,
+                     double              preference,
+                     const SConnNetInfo* net_info,
+                     SSERV_InfoCPtr      skip[],
+                     size_t              n_skip,
+                     int/*bool*/         external,
+                     const char*         arg,
+                     const char*         val)
+{
+    return s_Open(service,
+                  service  &&  (!*service  ||  strpbrk(service, "?*")), types,
+                  preferred_host, preferred_port, preference,
+                  net_info, skip, n_skip,
+                  external, arg, val,
+                  0/*info*/, 0/*host_info*/);
+}
+
+
+extern SSERV_Info* SERV_GetInfoSimple(const char* service)
+{
+    SConnNetInfo* net_info = ConnNetInfo_Create(service);
+    SSERV_Info* info = SERV_GetInfoP(service, fSERV_Any,
+                                     SERV_ANYHOST/*preferred_host*/,
+                                     0/*preferred_port*/, 0.0/*preference*/,
+                                     net_info, 0/*skip*/, 0/*n_skip*/,
+                                     0/*not external*/, 0/*arg*/, 0/*val*/,
+                                     0/*host_info*/);
+    ConnNetInfo_Destroy(net_info);
+    return info;
+}
+
+
+extern SSERV_Info* SERV_GetInfo(const char*         service,
+                                TSERV_Type          types,
+                                unsigned int        preferred_host,
+                                const SConnNetInfo* net_info)
+{
+    return SERV_GetInfoP(service, types,
+                         preferred_host,
+                         0/*preferred_port*/, 0.0/*preference*/,
+                         net_info, 0/*skip*/, 0/*n_skip*/,
+                         0/*not external*/, 0/*arg*/, 0/*val*/,
+                         0/*host_info*/);
+}
+
+
+extern SSERV_Info* SERV_GetInfoEx(const char*         service,
+                                  TSERV_Type          types,
+                                  unsigned int        preferred_host,
+                                  const SConnNetInfo* net_info,
+                                  SSERV_InfoCPtr      skip[],
+                                  size_t              n_skip,
+                                  HOST_INFO*          host_info)
+{
+    return SERV_GetInfoP(service, types,
+                         preferred_host,
+                         0/*preferred_port*/, 0.0/*preference*/,
+                         net_info, skip, n_skip,
+                         0/*not external*/, 0/*arg*/, 0/*val*/,
+                         host_info);
+}
+
+
+SSERV_Info* SERV_GetInfoP(const char*         service,
+                          TSERV_Type          types,
+                          unsigned int        preferred_host,
+                          unsigned short      preferred_port,
+                          double              preference,
+                          const SConnNetInfo* net_info,
+                          SSERV_InfoCPtr      skip[],
+                          size_t              n_skip,
+                          int/*bool*/         external,
+                          const char*         arg,
+                          const char*         val,
+                          HOST_INFO*          host_info)
+{
+    SSERV_Info* info;
     SERV_ITER iter = s_Open(service, 0/*not mask*/, types,
                             preferred_host, preferred_port, preference,
                             net_info, skip, n_skip,
                             external, arg, val,
                             &info, host_info);
-    if (iter  &&  iter->op  &&  !info) {
-        /* All LOCAL/DISPD searches end up here, but none LBSMD ones */
-        info = s_GetNextInfo(iter, host_info, 1/*internal*/);
-    }
+    assert(!info  ||  iter);
     SERV_Close(iter);
     return info;
 }
 
 
-SSERV_Info* SERV_GetInfo(const char*         service,
-                         TSERV_Type          types,
-                         unsigned int        preferred_host,
-                         const SConnNetInfo* net_info)
+extern SSERV_InfoCPtr SERV_GetNextInfoEx(SERV_ITER  iter,
+                                         HOST_INFO* host_info)
 {
-    return s_GetInfo(service, types,
-                     preferred_host, 0/*preferred_port*/, 0.0/*preference*/,
-                     net_info, 0/*skip*/, 0/*n_skip*/,
-                     0/*not external*/, 0/*arg*/, 0/*val*/,
-                     0/*host_info*/);
+    assert(!iter  ||  iter->op);
+    return iter ? s_GetNextInfo(iter, host_info, 0) : 0;
 }
 
 
-SSERV_Info* SERV_GetInfoEx(const char*          service,
-                           TSERV_Type           types,
-                           unsigned int         preferred_host,
-                           const SConnNetInfo*  net_info,
-                           const SSERV_InfoCPtr skip[],
-                           size_t               n_skip,
-                           HOST_INFO*           host_info)
+extern SSERV_InfoCPtr SERV_GetNextInfo(SERV_ITER iter)
 {
-    return s_GetInfo(service, types,
-                     preferred_host, 0/*preferred_host*/, 0.0/*preference*/,
-                     net_info, skip, n_skip,
-                     0/*not external*/, 0/*arg*/, 0/*val*/,
-                     host_info);
-}
-
-
-SSERV_Info* SERV_GetInfoP(const char*          service,
-                          TSERV_Type           types,
-                          unsigned int         preferred_host,
-                          unsigned short       preferred_port,
-                          double               preference,
-                          const SConnNetInfo*  net_info,
-                          const SSERV_InfoCPtr skip[],
-                          size_t               n_skip,
-                          int/*bool*/          external, 
-                          const char*          arg,
-                          const char*          val,
-                          HOST_INFO*           host_info)
-{
-    return s_GetInfo(service, types,
-                     preferred_host, preferred_port, preference,
-                     net_info, skip, n_skip,
-                     external, arg, val,
-                     host_info);
-}
-
-
-const SSERV_Info* SERV_GetNextInfoEx(SERV_ITER  iter,
-                                     HOST_INFO* host_info)
-{
-    return iter  &&  iter->op ? s_GetNextInfo(iter, host_info, 0) : 0;
-}
-
-
-const SSERV_Info* SERV_GetNextInfo(SERV_ITER iter)
-{
-    return iter  &&  iter->op ? s_GetNextInfo(iter, 0, 0) : 0;
+    assert(!iter  ||  iter->op);
+    return iter ? s_GetNextInfo(iter, 0,         0) : 0;
 }
 
 
 const char* SERV_MapperName(SERV_ITER iter)
 {
-    return iter  &&  iter->op ? iter->op->name : 0;
+    assert(!iter  ||  iter->op);
+    return iter ? iter->op->mapper : 0;
 }
 
 
@@ -532,27 +593,29 @@ const char* SERV_CurrentName(SERV_ITER iter)
 
 int/*bool*/ SERV_PenalizeEx(SERV_ITER iter, double fine, TNCBI_Time time)
 {
-    if (!iter  ||  !iter->op  ||  !iter->op->Feedback  ||  !iter->last)
+    assert(!iter  ||  iter->op);
+    if (!iter  ||  !iter->op->Feedback  ||  !iter->last)
         return 0/*false*/;
     return iter->op->Feedback(iter, fine, time ? time : 1/*NB: always != 0*/);
 }
 
 
-int/*bool*/ SERV_Penalize(SERV_ITER iter, double fine)
+extern int/*bool*/ SERV_Penalize(SERV_ITER iter, double fine)
 {
     return SERV_PenalizeEx(iter, fine, 0);
 }
 
 
-int/*bool*/ SERV_Rerate(SERV_ITER iter, double rate)
+extern int/*bool*/ SERV_Rerate(SERV_ITER iter, double rate)
 {
-    if (!iter  ||  !iter->op  ||  !iter->op->Feedback  ||  !iter->last)
+    assert(!iter  ||  iter->op);
+    if (!iter  ||  !iter->op->Feedback  ||  !iter->last)
         return 0/*false*/;
     return iter->op->Feedback(iter, rate, 0/*i.e.rate*/);
 }
 
 
-void SERV_Reset(SERV_ITER iter)
+extern void SERV_Reset(SERV_ITER iter)
 {
     if (!iter)
         return;
@@ -564,14 +627,14 @@ void SERV_Reset(SERV_ITER iter)
 }
 
 
-void SERV_Close(SERV_ITER iter)
+extern void SERV_Close(SERV_ITER iter)
 {
     size_t i;
     if (!iter)
         return;
     SERV_Reset(iter);
     for (i = 0;  i < iter->n_skip;  i++)
-        free(iter->skip[i]);
+        free((void*) iter->skip[i]);
     iter->n_skip = 0;
     if (iter->op) {
         if (iter->op->Close)
@@ -579,7 +642,7 @@ void SERV_Close(SERV_ITER iter)
         iter->op = 0;
     }
     if (iter->skip)
-        free(iter->skip);
+        free((void*) iter->skip);
     free((void*) iter->name);
     free(iter);
 }
@@ -590,7 +653,8 @@ int/*bool*/ SERV_Update(SERV_ITER iter, const char* text, int code)
     static const char used_server_info[] = "Used-Server-Info-";
     int retval = 0/*not updated yet*/;
 
-    if (iter  &&  iter->op  &&  text) {
+    assert(!iter  ||  iter->op);
+    if (iter  &&  text) {
         const char *c, *b;
         iter->time = (TNCBI_Time) time(0);
         for (b = text;  (c = strchr(b, '\n')) != 0;  b = c + 1) {
@@ -604,9 +668,9 @@ int/*bool*/ SERV_Update(SERV_ITER iter, const char* text, int code)
                 continue;
             memcpy(t, b, len);
             if (t[len - 1] == '\r')
-                t[len - 1] =  '\0';
+                t[len - 1]  = '\0';
             else
-                t[len] = '\0';
+                t[len    ]  = '\0';
             p = t;
             if (iter->op->Update  &&  iter->op->Update(iter, p, code))
                 retval = 1/*updated*/;
@@ -614,7 +678,7 @@ int/*bool*/ SERV_Update(SERV_ITER iter, const char* text, int code)
                 &&  isdigit((unsigned char) p[sizeof(used_server_info) - 1])) {
                 p += sizeof(used_server_info) - 1;
                 if (sscanf(p, "%u: %n", &d1, &d2) >= 1
-                    &&  (info = SERV_ReadInfoEx(p + d2, "")) != 0) {
+                    &&  (info = SERV_ReadInfoEx(p + d2, "", 0)) != 0) {
                     if (!s_AddSkipInfo(iter, "", info))
                         free(info);
                     else
@@ -628,52 +692,13 @@ int/*bool*/ SERV_Update(SERV_ITER iter, const char* text, int code)
 }
 
 
-static void s_PrintFirewallPorts(char* buf, size_t bufsize,
-                                 const SConnNetInfo* net_info)
-{
-    EFWMode mode = net_info ? (EFWMode) net_info->firewall : eFWMode_Legacy;
-    size_t  len, n;
-    unsigned int m;
-
-    assert(buf  &&  bufsize > 1);
-    switch (mode) {
-    case eFWMode_Legacy:
-        *buf = '\0';
-        return;
-    case eFWMode_Firewall:
-        memcpy(buf, "0", 2);
-        return;
-    default:
-        break;
-    }
-    len = 0;
-    for (n = m = 0; n < SizeOf(s_FWPorts); n++, m += sizeof(s_FWPorts[0])<<3) {
-        unsigned short p;
-        TNCBI_BigCount mask = s_FWPorts[n];
-        for (p = m;  mask;  p++, mask >>= 1) {
-            if (mask & 1) {
-                char port[10];
-                int  k = sprintf(port, " %hu" + !len, p);
-                if (len + k < bufsize) {
-                    memcpy(buf + len, port, k);
-                    len += k;
-                }
-                if (!p)
-                    break;
-            }
-        }
-    }
-    buf[len] = '\0';
-}
-
-
 static void s_SetDefaultReferer(SERV_ITER iter, SConnNetInfo* net_info)
 {
     char* str, *referer = 0;
 
-    if (strcasecmp(iter->op->name, "DISPD") == 0)
+    if (strcasecmp(iter->op->mapper, "DISPD") == 0)
         referer = ConnNetInfo_URL(net_info);
-    else if ((str = strdup(iter->op->name)) != 0) {
+    else if ((str = strdup(iter->op->mapper)) != 0) {
         const char* host = net_info->client_host;
         const char* args = net_info->args;
         const char* name = iter->name;
@@ -705,8 +730,8 @@ static void s_SetDefaultReferer(SERV_ITER iter, SConnNetInfo* net_info)
 
 char* SERV_Print(SERV_ITER iter, SConnNetInfo* net_info, int/*bool*/ but_last)
 {
-    static const char kClientRevision[] = "Client-Revision: %hu.%hu\r\n";
     static const char kAcceptedServerTypes[] = "Accepted-Server-Types:";
+    static const char kClientRevision[] = "Client-Revision: %u.%u\r\n";
     static const char kUsedServerInfo[] = "Used-Server-Info: ";
     static const char kNcbiFWPorts[] = "NCBI-Firewall-Ports: ";
     static const char kServerCount[] = "Server-Count: ";
@@ -714,8 +739,8 @@ char* SERV_Print(SERV_ITER iter, SConnNetInfo* net_info, int/*bool*/ but_last)
     static const char kSkipInfo[] = "Skip-Info-%u: ";
     static const char kAffinity[] = "Affinity: ";
     char buffer[128], *str;
+    TBSERV_TypeOnly t;
     size_t buflen, i;
-    TSERV_Type t;
     BUF buf = 0;
 
     /* Put client version number */
@@ -727,13 +752,14 @@ char* SERV_Print(SERV_ITER iter, SConnNetInfo* net_info, int/*bool*/ but_last)
         return 0;
     }
     if (iter) {
-        if (net_info && !net_info->http_referer && iter->op && iter->op->name)
+        assert(iter->op);
+        if (net_info  &&  !net_info->http_referer  &&  iter->op->mapper)
             s_SetDefaultReferer(iter, net_info);
         /* Accepted server types */
         buflen = sizeof(kAcceptedServerTypes) - 1;
         memcpy(buffer, kAcceptedServerTypes, buflen);
         for (t = 1;  t;  t <<= 1) {
-            if (iter->type & t) {
+            if (iter->types & t) {
                 const char* name = SERV_TypeStr((ESERV_Type) t);
                 size_t namelen = strlen(name);
                 if (!namelen  ||  buflen + 1 + namelen + 2 >= sizeof(buffer))
@@ -762,9 +788,11 @@ char* SERV_Print(SERV_ITER iter, SConnNetInfo* net_info, int/*bool*/ but_last)
                 return 0;
             }
         }
-        if (iter->type & fSERV_Firewall) {
+        if (iter->types & fSERV_Firewall) {
             /* Firewall */
-            s_PrintFirewallPorts(buffer, sizeof(buffer), net_info);
+            EFWMode mode
+                = net_info ? (EFWMode) net_info->firewall : eFWMode_Legacy;
+            SERV_PrintFirewallPorts(buffer, sizeof(buffer), mode);
             if (*buffer
                 &&  (!BUF_Write(&buf, kNcbiFWPorts, sizeof(kNcbiFWPorts)-1)  ||
                      !BUF_Write(&buf, buffer, strlen(buffer))                ||
@@ -775,9 +803,12 @@ char* SERV_Print(SERV_ITER iter, SConnNetInfo* net_info, int/*bool*/ but_last)
         }
         if (iter->pref  &&  (iter->host | iter->port)) {
             /* Preference */
-            buflen  = SOCK_HostPortToString(iter->host, iter->port,
-                                            buffer, sizeof(buffer));
-            buflen += sprintf(buffer + buflen, " %.2lf%%\r\n", iter->pref*1e2);
+            buflen = SOCK_HostPortToString(iter->host, iter->port,
+                                           buffer, sizeof(buffer));
+            buffer[buflen++] = ' ';
+            buflen = (size_t)(strcpy(NCBI_simple_ftoa(buffer + buflen,
+                                                      iter->pref * 100.0, 2),
+                                     "\r\n") - buffer) + 2/*"\r\n"*/;
             if (!BUF_Write(&buf, kPreference, sizeof(kPreference) - 1)  ||
                 !BUF_Write(&buf, buffer, buflen)) {
                 BUF_Destroy(buf);
@@ -809,7 +840,7 @@ char* SERV_Print(SERV_ITER iter, SConnNetInfo* net_info, int/*bool*/ but_last)
                 buflen = sizeof(kUsedServerInfo) - 1;
                 memcpy(buffer, kUsedServerInfo, buflen);
             } else
-                buflen = sprintf(buffer, kSkipInfo, (unsigned) i + 1); 
+                buflen = sprintf(buffer, kSkipInfo, (unsigned) i + 1);
             assert(buflen < sizeof(buffer) - 1);
             if (!BUF_Write(&buf, buffer, buflen)                ||
                 (namelen  &&  !BUF_Write(&buf, name, namelen))  ||
@@ -842,31 +873,8 @@ char* SERV_Print(SERV_ITER iter, SConnNetInfo* net_info, int/*bool*/ but_last)
 }
 
 
-/*
- * Note parameters' ranges here:
- * 0.0 <= pref <= 1.0
- * 0.0 <  gap  <= 1.0
- * n >= 2
- * Hence, the formula below always yields in a value from the range [0..1].
- */
-double SERV_Preference(double pref, double gap, size_t n)
-{
-    double spread;
-    assert(0.0 <= pref && pref <= 1.0);
-    assert(0.0 <  gap  && gap  <= 1.0);
-    assert(n >= 2);
-    if (gap >= pref)
-        return gap;
-    spread = 14.0/(n + 12.0);
-    if (gap >= spread/((double) n))
-        return pref;
-    else
-        return 2.0/spread*gap*pref;
-}
-
-
-unsigned short SERV_ServerPort(const char*  name,
-                               unsigned int host)
+extern unsigned short SERV_ServerPort(const char*  name,
+                                      unsigned int host)
 {
     SSERV_Info*    info;
     unsigned short port;
@@ -877,11 +885,11 @@ unsigned short SERV_ServerPort(const char*  name,
      */
     if (!host  ||  host == SERV_LOCALHOST)
         host = SOCK_GetLocalHostAddress(eDefault);
-    if (!(info = s_GetInfo(name, fSERV_Standalone | fSERV_Promiscuous,
-                           host, 0/*pref. port*/, -1.0/*latch host*/,
-                           0/*net_info*/, 0/*skip*/, 0/*n_skip*/,
-                           0/*not external*/, 0/*arg*/, 0/*val*/,
-                           0/*host_info*/))) {
+    if (!(info = SERV_GetInfoP(name, fSERV_Standalone | fSERV_Promiscuous,
+                               host, 0/*pref. port*/, -1.0/*latch host*/,
+                               0/*net_info*/, 0/*skip*/, 0/*n_skip*/,
+                               0/*not external*/, 0/*arg*/, 0/*val*/,
+                               0/*host_info*/))) {
         return 0;
     }
     assert(info->host == host);
