@@ -29,7 +29,7 @@
 *   
 * Version Creation Date: 9/94
 *
-* $Revision: 6.11 $
+* $Revision: 6.13 $
 *
 * File Description:  Object manager for feature definitions
 *
@@ -319,8 +319,10 @@ NLM_EXTERN FeatDefPtr LIBCALL FeatDefFindNext PROTO((FeatDefPtr curr, Uint1Ptr k
 #define FEATDEF_centromere 99
 #define FEATDEF_telomere 100
 #define FEATDEF_assembly_gap 101
+#define FEATDEF_regulatory 102
+#define FEATDEF_propeptide 103
 
-#define FEATDEF_MAX 102 /* size of array needed for featdef filter parameters */
+#define FEATDEF_MAX 104 /* size of array needed for featdef filter parameters */
 
 
 #ifdef __cplusplus
